/*
 * iSeries device support
 */

#include <epicsStdio.h>
#include <devCommonGpib.h>

/******************************************************************************
 *
 * The following define statements are used to declare the names to be used
 * for the dset tables.   
 *
 * A DSET_AI entry must be declared here and referenced in an application
 * database description file even if the device provides no AI records.
 *
 ******************************************************************************/
#define DSET_AI     deviSeriesAI
#define DSET_BO     deviSeriesBO
#define DSET_LI     deviSeriesLI
#define DSET_LO     deviSeriesLO

#include <devGpib.h> /* must be included after DSET defines */

#define TIMEOUT     2.0    /* I/O must complete within this time */
#define TIMEWINDOW  5.0   /* Wait this long after device timeout */

/******************************************************************************
 * Array of structures that define all GPIB messages
 * supported for this type of instrument.
 ******************************************************************************/

static struct gpibCmd gpibCmds[] = {
  /* Command 0, Get reading */
  {&DSET_AI, GPIBREAD, IB_Q_LOW, "*X01\r", "X01%lf", 0, 80,
                                                NULL, 0, 0, NULL, NULL, "\r" },
  /* Command 1, Get peak reading */
  {&DSET_AI, GPIBREAD, IB_Q_LOW, "*X02\r", "X02%lf", 0, 80,
                                                NULL, 0, 0, NULL, NULL, "\r" },
  /* Command 2, Get valley reading */
  {&DSET_AI, GPIBREAD, IB_Q_LOW, "*X03\r", "X03%lf", 0, 80,
                                                NULL, 0, 0, NULL, NULL, "\r" },
  /* 3 - 9 */ FILL, FILL, FILL, FILL, FILL, FILL, FILL,

  /* Command 10, Get reading configuration (from volatile memory) */
  {&DSET_LI, GPIBREAD, IB_Q_LOW, "*G08\r", "G08%lx", 0, 80,
                                                NULL, 0, 0, NULL, NULL, "\r" },
  /* Command 11, Put reading configuration (to volatile memory) */
  {&DSET_LO, GPIBWRITE, IB_Q_LOW, NULL, "*P08%2.2X\r", 80, 80,
                                                NULL, 0, 0, NULL, NULL, "\r" },
  /* Command 12, Read reading configuration (from non-volatile memory) */
  {&DSET_LI, GPIBREAD, IB_Q_LOW, "*R08", "R08%lx\r", 0, 80,
                                                NULL, 0, 0, NULL, NULL, "\r" },
  /* Command 13, Write reading configuration (to non-volatile memory) */
  {&DSET_LO, GPIBWRITE, IB_Q_LOW, NULL, "*W08%2.2X\r", 80, 80,
                                                NULL, 0, 0, NULL, NULL, "\r" },
  /* 14 - 19 */ FILL, FILL, FILL, FILL, FILL, FILL,

  /* Command 20, Read input type (from non-volatile memory) */
  {&DSET_LI, GPIBREAD, IB_Q_LOW, "*R07\r", "R07%lx", 0, 80,
                                                NULL, 0, 0, NULL, NULL, "\r" },
  /* Command 21, Write input type (to non-volatile memory) */
  {&DSET_LO, GPIBWRITE, IB_Q_LOW, NULL, "*W07%2.2X\r", 80, 80,
                                                NULL, 0, 0, NULL, NULL, "\r" },
  /* 22 - 29 */ FILL, FILL, FILL, FILL, FILL, FILL, FILL, FILL,

  /* Command 30, Reset controller */
  {&DSET_BO, GPIBWRITE, IB_Q_LOW, NULL, "*Z02\r", 80, 80,
                                                NULL, 0, 0, NULL, NULL, "\r" },
  /* 31 - 39 */ FILL, FILL, FILL, FILL, FILL, FILL, FILL, FILL, FILL,

  /* Command 40, Read colors (from non-volatile memory) */
  {&DSET_LI, GPIBREAD, IB_Q_LOW, "*R11\r", "R11%lx", 0, 80,
                                                NULL, 0, 0, NULL, NULL, "\r" },
  /* Command 41, Write colors (to non-volatile memory) */
  {&DSET_LO, GPIBWRITE, IB_Q_LOW, NULL, "*W11%2.2X\r", 80, 80,
                                                NULL, 0, 0, NULL, NULL, "\r" },
};

/* The following is the number of elements in the command array above.  */
#define NUMPARAMS sizeof(gpibCmds)/sizeof(struct gpibCmd)

/******************************************************************************
 * Initialize device support parameters
 *
 *****************************************************************************/
static long init_ai(int parm)
{
    if(parm==0) {
        devSupParms.name = "deviSeries";
        devSupParms.gpibCmds = gpibCmds;
        devSupParms.numparams = NUMPARAMS;
        devSupParms.timeout = TIMEOUT;
        devSupParms.timeWindow = TIMEWINDOW;
        devSupParms.respond2Writes = 0;
    }
    return(0);
}
